#include <QSqlError>
#include <QTreeWidgetItem>
#include <QMessageBox>
#include "AlarmHistoryTree.h"
#include "mysqlmodel.h"
#include <QHeaderView>
#include <QtDebug>
#include <QApplication>
#include <QCoreApplication>
#include <cumacros.h>
#include <cudata.h>
#include <qustring.h>

AlarmHistoryTree::AlarmHistoryTree(QWidget *parent) : QTreeView(parent) {
    m_sql_model = new MySqlModel();
    m_sql_model->setHeaderData(0, Qt::Horizontal, "Timestamp");
    setModel(m_sql_model);
}

void AlarmHistoryTree::applyFilter(AlarmFilter f)
{   
    int i, j;
    QList<QVariant> rowEls;
    if(_filter_enabled)
        removeFilter();

    int rowCount = m_sql_model->rowCount();
    int colCount = m_sql_model->columnCount();
    QModelIndex root = rootIndex();

    for(i = 0; i < rowCount; i++)  {
        rowEls.clear();
        for(j = 0; j < colCount; j++)
            rowEls << m_sql_model->data(m_sql_model->index(i, j),  Qt::DisplayRole);
        if(f.stat) {
            /* must not have been hidden previously */
            printf("AlarmHistoryTree.applyFilter: status %s row status %s\n", qstoc(f.status), qstoc(rowEls[STATUS].toString()));
            if(rowEls.size() > STATUS && rowEls[STATUS] != f.status && !isRowHidden(i, root))
                setRowHidden(i, root, true);
        }
    }
    _filter_enabled = true;

}

void AlarmHistoryTree::removeFilter() {
    int i;
    QModelIndex root = rootIndex();
    for(i = 0; i < m_sql_model->rowCount(); i++) {
        if(isRowHidden(i, root))
            setRowHidden(i, root, false);
    }
    _filter_enabled = false;
}

QAbstractItemModel *AlarmHistoryTree::model() const {
    return m_sql_model;
}

void AlarmHistoryTree::onFilterNameChanged(const QString &f) {
    for(int i = 0; i < m_sql_model->rowCount(); i++) {
        setRowHidden(i, QModelIndex(), !f.isEmpty() && !m_sql_model->data(m_sql_model->index(i, NAME), Qt::DisplayRole).toString().contains(f));
    }
}

void AlarmHistoryTree::onDbParamsReady(const CuData &p) {
    QString m;
    bool err;
    p["err"].to<bool>(err);
    if(err) {
        m = QuString(p, "msg");
        printf("AlarmHistoryTree::onDbParamsReady error ? %s\n", datos(p));
    }
    else {
        db = QSqlDatabase::addDatabase("QMYSQL");
        std::vector<std::string> pa = p["value"].toStringVector();
        if(pa.size() != 5)
            m = QString("HistoryDB property: expected 4 parameters (host, user, password, database, port), found %1").arg(pa.size());
        else {
            db.setHostName(pa[0].c_str());
            db.setUserName(pa[1].c_str());
            db.setPassword(pa[2].c_str());
            db.setDatabaseName(pa[3].c_str());
            db.setPort(QuString(pa[4]).toInt());
            if (!db.open())
                m = QString("Can't establish connection with db with params %s:\n%1").arg(p["value"].toString().c_str()).arg(db.lastError().text());
            else
                printf("AlarmHistoryTree::onDbParamsReady: successfully opened db\n");
        }
    }
    if(!m.isEmpty())
        QMessageBox::critical(0, "Error", m);

}

void AlarmHistoryTree::executeQuery(QString &q) {
    if (!db.isOpen())
    {
        if (!db.open())
        {
            QMessageBox::critical(0, "Error", "Can't establish connection with db Alarm on host log");
            return;
        }
    }
    m_sql_model->setQuery(q);
    QSqlError error = m_sql_model->lastError();
    if (error.type() != QSqlError::NoError)
    {
        QMessageBox::critical(0, "Database Error", QString("Error reading from Db:\n") + error.text() + "\nreconnecting");
        db.close();
        if (!db.open())
        {
            QMessageBox::critical(0, "Error", "Can't establish connection with db FastInterlock on host log");
            return;
        }
    }
    for(int i = 0; i < header()->count(); i++)
    {
        resizeColumnToContents(i);
        header()->setSortIndicator(i, Qt::AscendingOrder);
    }
}

